//===- iterator.cpp - iterator over function, basic block & instruction ---===//
//
//===----------------------------------------------------------------------===//
//
// This file implements iterator pass like LLVM "Hello World" pass described
// in docs/WritingAnLLVMPass.html
//
//===----------------------------------------------------------------------===//

#include "llvm/Pass.h"
#include "llvm/Module.h"
#include "llvm/ADT/Statistic.h"
#include "llvm/Support/raw_ostream.h"
#include "llvm/Target/TargetData.h"

using namespace llvm;

namespace
{

  struct MyIterator : public ModulePass
  {
    static char ID;

    MyIterator() : ModulePass(ID) {}

    virtual bool runOnModule(Module &M)
    {
      for (Module::iterator F = M.begin(), E = M.end(); F != E; ++F)
      {
        if (F->isDeclaration()) continue;

        for (Function::iterator BB = F->begin(), E = F->end(); BB != E; ++BB)
        {
          MyIterator::runOnBasicBlock(BB);
        }
      }

      return false;
    }


    virtual bool runOnBasicBlock(Function::iterator &BB)
    {
      errs() << "========BB===========\n";

      for (BasicBlock::iterator BI = BB->begin(), BE = BB->end(); BI != BE; ++BI)
      { 
        errs() << "BI: " << BI->getOpcodeName() << "\n";
        errs() << *BI << "\n";
      }

      errs() << "========BB===========\n\n";
      return false;
    }

  private:
    virtual void getAnalysisUsage(AnalysisUsage &AU) const
    {
      AU.addRequired<TargetData>();
    }

  };

}

char MyIterator::ID = 0;
static RegisterPass<MyIterator> X("iterator", "Iterator Pass");

